!===============================================================================
! Usefull functions (perfect gas law)
!===============================================================================
!===============================================================================
subroutine sound_velocity(gama,pres,rho, sound)

implicit none

! Arguments

double precision gama, pres, rho, sound

sound = sqrt(gama*pres/rho)

end subroutine sound_velocity
!===============================================================================
subroutine aeras_law(gama,mach,sec,dsec)

implicit none

! Arguments

double precision gama, mach, sec, dsec

sec  = 1.d0/mach*( &
         2.d0/(gama+1.d0)*(1+0.5d0*(gama-1.d0)*mach**2.d0) &
                )**((gama+1.d0)/(2.d0*(gama-1.d0)))
dsec = sec/mach*(  &
         2.d0*(mach**2-1.d0)/(2.d0+(gama-1.d0)*mach**2)    &
                )

return
end subroutine aeras_law
!===============================================================================
subroutine pres_isent(gama,mach,pres)

implicit none

! Arguments

double precision gama, mach, pres

pres = (1.d0+0.5d0*(gama-1.d0)*mach**2.d0)**(gama/(gama-1.d0))

return
end subroutine pres_isent
!===============================================================================
subroutine invpres_isent(gama,pres,mach)

implicit none

! Arguments

double precision gama, mach, pres

mach = pres**((gama-1.d0)/gama)-1.d0
if (mach.ge.0.d0) then
  mach = sqrt((2.d0/(gama-1.d0))*mach)
else
  write(*,*) 'error: the isentropic pressure ratio should be greater than one', &
             ' to get the Mach number from it'
  stop
endif

return
end subroutine invpres_isent
!===============================================================================
subroutine rho_isent(gama,mach,rho)

implicit none

! Arguments

double precision gama, mach, rho

rho = (1.d0+0.5d0*(gama-1.d0)*mach**2.d0)**(1.d0/(gama-1.d0))

return
end subroutine rho_isent
!===============================================================================
subroutine temp_isent(gama,mach,temp)

implicit none

! Arguments

double precision gama, mach, temp

temp = 1.d0+0.5d0*(gama-1.d0)*mach**2.d0

return
end subroutine temp_isent
!===============================================================================
subroutine pres_shock(gama,mach,pres)

implicit none

! Arguments

double precision gama, mach, pres

pres = 1.d0+(2.d0*gama/(gama+1.d0))*(mach**2-1.d0)

return
end subroutine pres_shock
!===============================================================================
subroutine mach_shock(gama,mach1,mach2)

implicit none

! Arguments

double precision gama, mach1, mach2

mach2 = (1.d0+0.5d0*(gama-1.d0)*mach1**2.d0)/(gama*mach1**2.d0-0.5d0*(gama-1.d0))
if (mach2.ge.0.d0) then
  mach2 = sqrt(mach2)
else
  write(*,*) 'error: the function giving the Mach number after the shock', &
             ' is non positive'
  stop
endif

return
end subroutine mach_shock
!===============================================================================
subroutine presgen_shock(gama,mach,pres,dpres)

implicit none

! Arguments

double precision gama, mach, pres, dpres

pres = (1.d0+((2.d0*gama)/(gama+1.d0))*(mach*mach-1.d0))**(-1.d0/(gama-1.d0))* &
       (1.d0-(2.d0/(gama+1.d0))*(1.d0-1.d0/(mach*mach)))**(-gama/(gama-1.d0))
dpres = -(4.d0*gama/(gama**2.d0-1.d0))*mach*pres*                            &
        (1.d0/(1.d0+(2.d0*gama/(gama+1.d0))*(mach**2.d0-1.d0))               &
         +1.d0/(mach**4.d0*(1.d0-(2.d0/(gama+1.d0))*(1.d0-1.d0/mach**2.d0)))                                             &
        )

return
end subroutine presgen_shock
!===============================================================================
subroutine newton(nitmx,iwarni,gama,mach0,sec0,mach)

implicit none

! Arguments

integer          nitmx, iwarni
double precision gama, mach0

! Variables locales
integer          inewt
double precision eps, mach, sec0, phi, dphi, f

! Initialisation du newton
eps  = 1.d-14
mach = mach0

do inewt = 1, nitmx

! Declaration de la fonction et de sa derivee
  call aeras_law(gama,mach,f,dphi)
  phi = f-sec0

! Calcul du point suivant

mach = mach - phi/dphi
if (abs(phi) .lt. eps) then
  if (iwarni.eq.1) then
    write(*,*) 'newton: it = ', inewt, 'phi = ', phi
  endif
  return
elseif (inewt.eq.nitmx) then
  write(*,*) ' --- error: non convergence of the newton algorithm ---', inewt, phi
  exit
endif

if (iwarni.eq.1) then
  write(*,*) 'newton: it = ', inewt, 'phi = ', phi
endif

enddo

end subroutine newton
!===============================================================================
subroutine newton2(nitmx,iwarni,gama,mach0,pres0,mach)

implicit none

! Arguments

integer          nitmx, iwarni
double precision gama, mach0

! Variables locales
integer          inewt
double precision eps, mach, pres0, phi, dphi, f

! Initialisation du newton
eps  = 1.d-15
mach = mach0

do inewt = 1, nitmx

! Declaration de la fonction et de sa derivee
  call presgen_shock(gama,mach,f,dphi)
  phi = f-pres0

! Calcul du point suivant

mach = mach - phi/dphi
if (abs(phi) .lt. eps) then
  if (iwarni.eq.1) then
    write(*,*) 'newton: it = ', inewt, 'phi = ', phi
  endif
  return
elseif (inewt.eq.nitmx) then
  write(*,*) ' --- error: non convergence of the newton algorithm ---', inewt, phi
  exit
endif

if (iwarni.eq.1) then
  write(*,*) 'newton: it = ', inewt, 'phi = ', phi
endif

enddo

end subroutine newton2
!===============================================================================
subroutine dichotomy(iwarni, gama, mach1, mach2, pres0, mach)

implicit none

! Arguments

integer          iwarni
double precision gama, pres0, mach, mach1, mach2

integer          isweep
double precision presa, presb, presmid
double precision macha, machb, machmid
double precision prod, proda, prodb
double precision rvoid, eps

! Initialisation
eps     = 1.d-15
isweep  = 1
presmid = 1.d12
macha = mach1
machb = mach2
call presgen_shock(gama,macha,presa,rvoid)
call presgen_shock(gama,machb,presb,rvoid)
prod = (presa-pres0)*(presb-pres0)
if (prod.ge.0.d0) then
  write(*,*) '--- error: the initialisation of the dichotomy should be checked ---'
  stop
endif

do while (abs(presmid-pres0).gt.eps)

  machmid = 0.5d0*(macha + machb)
  call presgen_shock(gama,macha,presa,rvoid)
  call presgen_shock(gama,machb,presb,rvoid)
  call presgen_shock(gama,machmid,presmid,rvoid)
  proda = (presmid-pres0)*(presa-pres0)
  prodb = (presmid-pres0)*(presb-pres0)
  if (proda.le.0.d0) then
    macha = macha
    machb = machmid
  elseif (prodb.le.0.d0) then
    macha = machmid
    machb = machb
  endif
  if (iwarni.eq.1) then
    write(*,*) 'dichotomy, it = ', isweep, 'residu = ', abs(presmid-pres0)
  endif
  isweep = isweep + 1

enddo
if (iwarni.eq.1) then
  write(*,*) 'dichotomy, it = ', isweep, 'residu = ', abs(presmid-pres0)
endif
mach = machmid
end subroutine dichotomy
